package org.zeroturnaround.javarebel.integration.jboss4;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

public class RepositoryClassLoaderCBP extends JavassistClassBytecodeProcessor {
	public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
		cp.importPackage("java.util");
		cp.importPackage("java.net");
		cp.importPackage("org.zeroturnaround.javarebel");
		cp.importPackage("org.zeroturnaround.javarebel.support");
		cp.importPackage("org.jboss.classloader.plugins");
		cp.importPackage("org.zeroturnaround.javarebel.integration.util");		
		
		ctClass.addInterface(cp.get(ClassResourceSource.class.getName()));

		CtConstructor[] constructors = ctClass.getConstructors();
		for (int i = 0; i < constructors.length; i++) {
			CtConstructor constructor = constructors[i];
			constructor.insertAfter("IntegrationFactory.getInstance().registerClassLoader($0, (ClassResourceSource) $0);");
		}

		CtMethod loadMethod = ctClass.getMethod("findClass", "(Ljava/lang/String;)Ljava/lang/Class;");
		loadMethod.insertBefore(
        "{   synchronized ($0) {" + 
        "      Class result =" + 
        "        findLoadedClass($1);" + 
        "      if (result != null)" + 
        "        return result;" + 
        "      result = " + 
        "        org.zeroturnaround.javarebel.IntegrationFactory.getInstance()" + 
        "          .findReloadableClass($0, $1);" + 
        "      if (result != null)" + 
        "        return result;" + 
        "    }}");
		
    ctClass.addMethod(CtNewMethod.make(
        "protected URL findResource(String name) {\n" +
        "      Integration integration = IntegrationFactory.getInstance();" +
        "      if (integration.isResourceReplaced($0, name))" +
        "        return integration.findResource($0, name);" + 
        "    return super.findResource(name);" +
        "}", ctClass)); 		
		
		CtMethod getResourcesMethod = ctClass.getDeclaredMethod("findResourcesLocally");
		getResourcesMethod.insertBefore(
				"{" + 
        "      Integration integration = IntegrationFactory.getInstance();" +
        "      if (integration.isResourceReplaced($0, $1))" +
        "        return integration.findResources($0, $1);\n" +
        "}");		
				
		ctClass.addMethod(CtNewMethod.make(
		    "public Resource getClassResource(String className) {"
				+ " final String resourcePath = className.replace('.', '/') + \".class\";"
				+ " URL url = findResource(resourcePath);"
				+ " if (url == null) return null;\n"
				+ " return ResourceUtil.asResource(url);" +
				"}", ctClass));
		
		ctClass.addMethod(CtNewMethod.make(
		    "public Resource getLocalResource(String resourcePath) {"
				+ " URL url = findResource(resourcePath);"
				+ " if (url == null) return null;\n"
				+ " return ResourceUtil.asResource(url);" +
			  "}", ctClass));	  
		
		ctClass.addMethod(CtNewMethod.make("public Resource[] getLocalResources(String resourcePath) {"
				+ " URL[] urls = ResourceUtil.toURLs(findResourcesLocally(resourcePath));\n"
				+ " Resource[] result = new Resource[urls.length];\n" + 
					"    for (int i = 0; i < urls.length; i++) {\n" + 
					"      result[i] = ResourceUtil.asResource(urls[i]);\n" +  
					"    }\n" + 
					"    return result;" +
				"}", ctClass));
	}
}
